#include <Arduino.h>
#include <avr/io.h>
#include <LiquidCrystal_I2C.h>
#include "Rocket.h"
#include "Terrain.h"
#include "bitmaps.h"

const int MAXROW=16;
const int MAXCOLUMN=2;
const int button = 0;
const int Vx = 1;
const int Vy=2;

LiquidCrystal_I2C lcd(0x27,MAXROW,MAXCOLUMN);
Rocket rocket;
Terrain terrain;

void updateLCD(bool terrainChange);
void checkJoystick();
bool updateTerrain();
bool checkCollision();
bool messageDisplayed = false;

unsigned long gameStartTime=millis();
int score = 0;
int buttonValue = 0;


void setup() {
  pinMode(Vx,INPUT);
  pinMode(Vy,INPUT);
  pinMode(button,INPUT_PULLUP);
  lcd.init();
  lcd.clear();         
  lcd.backlight();  
  lcd.createChar(0,rocketChar);
  lcd.createChar(1,blockChar);

  lcd.setCursor(0,0);
  lcd.write(0);


  Serial.begin(9600);
  randomSeed(analogRead(10));

}



void loop() { 

 if (score == 0) {

    if (checkCollision()==false){
      checkJoystick();
      bool terrainChanged = updateTerrain();
      updateLCD(terrainChanged);
    } 
    else { 
      if (messageDisplayed == false) { 
        messageDisplayed = true;
        unsigned long gameEndTime=millis();
        score = int((gameEndTime - gameStartTime) / 1000);
        lcd.clear();
        lcd.print("SCORE: ");
        lcd.print(score);
      }

    } 
  }
}

void updateLCD(bool terrainChanged){

  //update rocket
  if (rocket.getPreviousColumn() != rocket.getColumn() || rocket.getPreviousRow() != rocket.getRow()){
    lcd.setCursor(rocket.getPreviousRow(),rocket.getPreviousColumn());
    lcd.print(" ");
    lcd.setCursor(rocket.getRow(),rocket.getColumn());
    lcd.write(0);
    rocket.resetPosition();
  }

  if(terrainChanged==true){
    for (int i=0; i<=terrain.getLength();i++){
      for (int j=0;j<=terrain.getWidth();j++){
        if (terrain.getPosition(i,j)==1){
          lcd.setCursor(i,j);
          lcd.write(1);
        } 
        else if (terrain.getPosition(i,j)==0){
          if (i != rocket.getRow() || j != rocket.getColumn()) {
            lcd.setCursor(i,j);
            lcd.print(" ");
          }
        }
      }
    }
  }

}

bool updateTerrain(){

    static unsigned long lastGenObstTime=0;
    unsigned long currentGenObstTime=millis();
    if(currentGenObstTime-lastGenObstTime>5000){
        terrain.generateObstacle();
        lastGenObstTime=currentGenObstTime;
        return(true);
    }

    static unsigned long lastMoveObstTime=0;
    unsigned long currentMoveObstTime=millis();
    if(currentMoveObstTime-lastMoveObstTime>2000){
      terrain.moveObstacle();
      lastMoveObstTime=currentMoveObstTime;
      return true;
    }

    return false;
}

void checkJoystick(){
  static unsigned long lastMoveTime=0;
  unsigned long currentMoveTime=millis();

  if(currentMoveTime-lastMoveTime>250){
    if (analogRead(Vx)<400){
      rocket.moveRight();
    }else if(analogRead(Vx)>700){
      rocket.moveLeft();
    }else if (analogRead(Vy)<400){
      rocket.moveUp();
    }else if (analogRead(Vy)>700){
      rocket.moveDown();
    }
    lastMoveTime=currentMoveTime;
   
  }

}

bool checkCollision(){
  if(terrain.getPosition(rocket.getRow(),rocket.getColumn())==1){
    return true;
  }else{
    return false;
  }
}